@@ -1,5 +1,7 @@
 #!/bin/bash
 
+set -e
+
 DEPS=(
     # Setup
     wget
@@ -25,10 +27,8 @@ DEPS=(
     libzstd-dev
     lz4
     mawk
-    meson
     perl
     pkgconf
-    python3-dev
     systemtap-sdt-dev
     tcl-dev
     uuid-dev
@@ -40,14 +40,17 @@ DEPS=(
     # Test
     libipc-run-perl
     # Test pg_tde
-    python3-pykmip
     libhttp-server-simple-perl
+    lcov
+    # Run pgperltidy
+    perltidy
 )
 
 sudo apt-get update
 sudo apt-get install -y ${DEPS[@]}
 
-bash -c "$(wget -O - https://apt.llvm.org/llvm.sh)"
+sudo bash -c "$(wget -O - https://apt.llvm.org/llvm.sh)"
+pip3 install meson pykmip cryptography setuptools wheel
 
 # Vault
 wget -O - https://apt.releases.hashicorp.com/gpg | sudo tee /etc/apt/keyrings/hashicorp-archive-keyring.asc
