@@ -1,20 +1,12 @@
-/*-------------------------------------------------------------------------
- *
- * pg_tde_guc.c
- *	  GUC variables for pg_tde
- *
- *
- * IDENTIFICATION
- *	  src/pg_tde_guc.c
- *
- *-------------------------------------------------------------------------
+/*
+ * GUC variables for pg_tde
  */
 
-#include "pg_tde_guc.h"
 #include "postgres.h"
+
 #include "utils/guc.h"
 
-#ifndef FRONTEND
+#include "pg_tde_guc.h"
 
 bool		AllowInheritGlobalProviders = true;
 bool		EncryptXLog = false;
@@ -35,7 +27,6 @@ TdeGucInit(void)
 							 NULL	/* show_hook */
 		);
 
-#ifdef PERCONA_EXT
 	DefineCustomBoolVariable("pg_tde.wal_encrypt",	/* name */
 							 "Enable/Disable encryption of WAL.",	/* short_desc */
 							 NULL,	/* long_desc */
@@ -60,7 +51,4 @@ TdeGucInit(void)
 							 NULL	/* show_hook */
 		);
 
-#endif
 }
-
-#endif
