@@ -34,6 +34,7 @@
 #include "storage/checksum_impl.h"
 
 #ifdef PERCONA_EXT
+#include "pg_tde.h"
 #include "access/pg_tde_fe_init.h"
 #include "access/pg_tde_tdemap.h"
 #endif
@@ -137,10 +138,9 @@ pg_tde_init(const char *datadir)
 static bool
 is_pg_tde_encypted(Oid spcOid, Oid dbOid, RelFileNumber relNumber)
 {
-	RelFileLocator locator = {.spcOid = spcOid, dbOid = dbOid,.relNumber = relNumber};
-	RelFileLocatorBackend rlocator = {.locator = locator,.backend = INVALID_PROC_NUMBER};
+	RelFileLocator locator = {.spcOid = spcOid,.dbOid = dbOid,.relNumber = relNumber};
 
-	return GetSMGRRelationKey(rlocator) != NULL;
+	return pg_tde_has_smgr_key(locator);
 }
 #endif
 
